@@ -79,39 +79,64 @@ class DataPoint():
 class DenMune():
    
     def __init__ (self,
-                  train_data, test_data=None,
-                  train_labels=None, test_labels=None, 
+                  train_data=None, test_data=None,
+                  train_truth=None, test_truth=None, 
                   file_2d ='_temp_2d', k_nearest=1, 
                   rgn_tsne=False, prop_step=0,
-                  delimiter=',', verpose=True
+                  verpose=True
                   ):     
          
+        if train_data is None:
+           raise Exception("No data is provided. At least train data should be provided")
+        else:
+           self.data_indicator = 1
+        
+        if train_truth is not None:
+           self.data_indicator += 2
+        
+        if test_data is not None:
+           self.data_indicator += 4
+           
+        if test_truth is not None:
+           self.data_indicator += 8  
+
+        """
+        if  self.data_indicator <= 7: # train_truth is given
+           self.data_type = 'train'
+        elif  self.data_indicator <= 11: # test data is given
+           self.data_type = 'test'
+        elif  self.data_indicator == 15: # test_truth is given
+           self.data_type = 'augmented'   
+        """
+
         self.analyzer = {}
         self.analyzer['n_points'] = {}
-
         if isinstance(train_data, pd.DataFrame):
             train_data = train_data.to_numpy()
+            train_data = train_data.copy(order='C')
         if isinstance(test_data, pd.DataFrame):
             test_data = test_data.to_numpy()
-        if isinstance(train_labels, pd.Series):
-            train_labels = train_labels.to_numpy()
-        if isinstance(test_labels, pd.Series):
-            test_labels = test_labels.to_numpy()
+            test_data = test_data.copy(order='C')
+        if isinstance(train_truth, pd.Series):
+            train_truth = train_truth.to_numpy()
+            train_truth = train_truth.copy(order='C')
+        if isinstance(test_truth, pd.Series):
+            test_truth = test_truth.to_numpy()
+            test_truth = test_truth.copy(order='C')
            
         self.train_sz = len (train_data)
-
+   
         if test_data is not None:
             data = np.append(train_data, test_data, axis = 0)
             self.test_sz= len (test_data)
         else:
           self.test_sz= 0
           data = train_data
-        
-        
-        if test_labels is not None:
-            self.labels_true = np.append(train_labels, test_labels, axis = 0)
+
+        if test_truth is not None:
+            self.labels_truth = np.append(train_truth, test_truth, axis = 0)
         else:
-            self.labels_true = train_labels
+            self.labels_truth = train_truth
           
         self.analyzer["n_points"]["size"] = len(data) #data.shape[0] # this will changed in preplot when we plot only train or test data
         
@@ -126,10 +151,13 @@ class DenMune():
         self.analyzer["n_points"]["weak"]["failed to merge"] = 0
         self.analyzer["n_points"]["dim"] = data.shape[1]
         self.analyzer["validity"] = {}
+        self.analyzer["validity"]['train'] = {}
+        self.analyzer["validity"]['test'] = {}
+        self.analyzer["validity"]['augmented'] = {}
         self.analyzer["n_clusters"] = {}
         self.analyzer["n_clusters"]["actual"] = 0
         self.analyzer["n_clusters"]["detected"] = 0
-        self.delimiter = delimiter
+        #self.delimiter = delimiter
         self.debuger = {}
        
         if data.shape[1] != 2 and file_2d == '_temp_2d':
@@ -138,14 +166,14 @@ class DenMune():
             self.generate_tsne(data, 2, file_2d='_temp_2d')
             end = time.time()
             self.analyzer["exec_time"]["t_SNE"] = end-start
-            data = genfromtxt(file_2d , delimiter=self.delimiter)
+            data = genfromtxt(file_2d , delimiter=',')
         elif data.shape[1] != 2 and file_2d != '_temp_2d':    
             if  not os.path.isfile(file_2d) or rgn_tsne == True:
                 start = time.time()
                 self.generate_tsne(data, 2, file_2d)
                 end = time.time()
                 self.analyzer["exec_time"]["t_SNE"] = end-start
-            data = genfromtxt(file_2d , delimiter=self.delimiter)
+            data = genfromtxt(file_2d , delimiter=',')
 
         start_time = time.time()
         
@@ -159,10 +187,9 @@ class DenMune():
         self.dp_dim = self.data.shape[1]
         self.k_nearest = k_nearest
         self.dp_dis = []
-        self.train_labels = train_labels
-        self.test_labels = test_labels
+        self.train_truth = train_truth
+        self.test_truth = test_truth
 
-                   
         self.DataPoints = []
         self.ClassPoints = {}
         self.KernelPoints = []
@@ -187,21 +214,20 @@ class DenMune():
     def kd_NGT(self):
         
         if len(self.dp_dis) == 0:
-                    
+                              
             ngtpy.create(b"tmp", self.dp_dim)
             index = ngtpy.Index(b"tmp")
             index.batch_insert(self.data)
             index.save()
 
             k= self.k_nearest
-
             start = time.time()
             self.dp_dis = []
             for i in range(self.dp_count):
                 query = self.data[i]
                 result = index.search(query, k+1) [1:] #we skip first distance from a point to itself 
                 self.dp_dis.append(result)
-
+                
             end =  time.time()
             self.analyzer["exec_time"]["NGT"] = end-start
           
@@ -497,7 +523,7 @@ class DenMune():
 
     def fit_predict(self,
                     validate=True, show_plots=True,
-                    show_noise=True, data_type='train',
+                    show_noise=True, data_type=None,
                     ):
         validity_scores = []
         solution_file = 'solution.txt'
@@ -508,43 +534,70 @@ class DenMune():
         pred_list = []
         for dp in self.DataPoints:
             pred_list.append(dp.class_id)
-            
+
         with open(solution_file, 'w') as f:
             f.writelines("%s\n" % pred for pred in pred_list)
      
         labels_dic = {}
-        self.train_labels = pred_list [:self.train_sz]
-        self.test_labels = pred_list[self.train_sz:]
-        labels_dic ['train'] = self.train_labels
-        labels_dic ['test'] = self.test_labels
-
-        if validate and self.train_labels is not None:
-          validity_scores = self.validate_Clusters()
-          
-        if show_plots:
-           self.plot_clusters(show_plots=True, show_noise=show_noise, data_type=data_type)
+        self.train_pred = pred_list [:self.train_sz]
+        self.test_pred = pred_list[self.train_sz:]
+        labels_dic ['train'] = self.train_pred
+        labels_dic ['test'] = self.test_pred
+        
+        if self.test_data is not None:
+            self.labels_pred = np.append(self.train_pred, self.test_pred, axis = 0)
         else:
-          self.plot_clusters(show_plots=False, show_noise=show_noise, data_type=data_type)
+            self.labels_pred = self.train_pred 
 
-        if self.verpose:
-            self.show_Analyzer(self.analyzer, root="DenMune Analyzer")
-        
-           
 
-        return labels_dic, validity_scores
+        if self.data_indicator >= 3: 
+            print ("Plotting dataset Groundtruth")
+            self.plot_clusters(show_plots=show_plots, show_noise=show_noise, data_type='ground')
+
+        if validate and self.data_indicator >= 1: 
+
+           if self.data_indicator >= 3:
+               validity_scores = self.validate_Clusters(data_type='train') 
+         
+           print ('Plotting train data')  
+           self.plot_clusters(show_plots=show_plots, show_noise=show_noise, data_type='train')
+           if self.verpose:
+               self.show_Analyzer(root='Validating train data')   
+
+           if self.data_indicator == 15:
+               validity_scores = self.validate_Clusters(data_type='test') 
+               if self.verpose:
+                   self.show_Analyzer(self.analyzer['validity']['test'], root='Validating test data')
+          
+           if self.data_indicator > 3:
+               print ('Plotting test data')  
+               self.plot_clusters(show_plots=show_plots, show_noise=show_noise, data_type='test')
+          
+           if self.data_indicator == 15:
+               validity_scores = self.validate_Clusters(data_type='augmented') 
+               if self.verpose:
+                   self.show_Analyzer(self.analyzer['validity']['augmented'], root='Validating augmented data (train & test)')
+
+           if self.data_indicator > 3: 
+               print ('Plotting augmented data (train & test)')  
+               self.plot_clusters(show_plots=show_plots, show_noise=show_noise, data_type='augmented')
+         
+        return labels_dic, self.analyzer['validity']
         
-       
+
 
     def match_Labels(self):
        
-        labels_true = self.labels_true
-        
-        if self.test_labels is not None:
-            self.labels_pred = np.append(self.train_labels, self.test_labels, axis = 0)
+        labels_true = self.labels_truth
+
+        if isinstance(self.labels_pred, np.ndarray):
+           #labels_pred = np.array(self.labels_pred, dtype=np.int64)
+           labels_pred = self.labels_pred.tolist()
         else:
-            self.labels_pred = self.labels_pred
-        labels_pred = self.labels_pred.tolist()
+           labels_pred = self.labels_pred
+
         
+
         pred_set = set(labels_pred) 
         index = []
         x = 1
@@ -577,17 +630,33 @@ class DenMune():
 
         labels_pred = np.array(labels_pred, dtype=np.int64)
         self.labels_pred = labels_pred
-        self.train_labels = labels_pred[:self.train_sz]
-        self.test_labels = labels_pred[self.train_sz:]
+        self.train_pred = labels_pred[:self.train_sz]
+        self.test_pred = labels_pred[self.train_sz:]
 
 
         return labels_pred
     
     
-    def validate_Clusters(self):
+    def validate_Clusters(self, data_type=None):
+        
+        labels_true = self.labels_truth
+        if data_type == 'train':
+           labels_true = labels_true[:self.train_sz] 
+        elif data_type == 'test':
+           labels_true = labels_true[self.train_sz:]
+        elif data_type == 'augmented':
+           # keep it as it
+           0 == 0 
+
+        labels_pred = self.match_Labels() 
+        if data_type == 'train':
+           labels_pred = labels_pred[:self.train_sz] 
+        elif data_type == 'test':
+           labels_pred = labels_pred[self.train_sz:]
+        elif data_type == 'augmented':
+           # keep it as it
+           0 == 0 
         
-        labels_true = self.labels_true
-        labels_pred = self.match_Labels()
         self.analyzer["n_clusters"]["actual"] = len(np.unique(labels_true))
         
         # Score the clustering
@@ -649,52 +718,64 @@ class DenMune():
         
 
        # val = [acc, f1, nmi, ami, ari, homogeneity, completeness]
-        self.analyzer["validity"]  =  validity
-        self.analyzer["n_points"]["weak"]["succeeded to merge"] = self.analyzer["n_points"]["weak"]["all"] - self.analyzer["n_points"]["noise"]["type-2"]
-        self.analyzer["n_points"]["weak"]["failed to merge"] = self.analyzer["n_points"]["noise"]["type-2"]   
-                                    
-        #if self.verpose:
-        #    self.show_Analyzer(self.analyzer, root="DenMune Analyzer")
-        
-       
-        return validity  
+        self.analyzer["validity"][data_type] =  validity
+        #self.analyzer["validity"] =  validity
+    
+        return self.analyzer["validity"][data_type]  
     
         
         
-    def preplot_Clusters(self, data_type='train'):
+    def preplot_Clusters(self, data_type=None):
         
         self.analyzer["n_points"]["size"] = self.dp_count
         if data_type == 'test':
-          labels = self.test_labels
-          self.analyzer["n_points"]["plot_size"] = self.test_sz
+           self.analyzer["n_points"]["plot_size"] = self.test_sz
         elif data_type == 'train':
-          labels = self.train_labels
-          self.analyzer["n_points"]["plot_size"] = self.train_sz
+           self.analyzer["n_points"]["plot_size"] = self.train_sz
         elif data_type == 'augmented':
-          labels = list(itertools.chain(self.train_labels, self.test_labels))
-          self.analyzer["n_points"]["plot_size"] = self.dp_count
+           self.analyzer["n_points"]["plot_size"] = self.dp_count
+
+        
+        if data_type == 'ground':
+           labels = self.labels_truth
+           labels = np.array(labels, dtype=np.int64)
+        else:
+            labels = self.labels_pred  
+            if data_type == 'train':
+                labels = labels[:self.train_sz]
+            elif data_type == 'test':
+                labels = labels[self.train_sz:]
+            elif data_type == 'augmented':
+                #nothing to do
+                0 == 0
 
-        #labels = np.array(labels, dtype=np.int64)
-             
         noise_1 = list(labels).count(-1)
         self.analyzer["n_points"]["noise"]["type-1"] = noise_1
         
         noise_2 = list(labels).count(-2)
         self.analyzer["n_points"]["noise"]["type-2"]  = noise_2
         
+        self.analyzer["n_points"]["weak"]["succeeded to merge"] = self.analyzer["n_points"]["weak"]["all"] - self.analyzer["n_points"]["noise"]["type-2"]
+        self.analyzer["n_points"]["weak"]["failed to merge"] = self.analyzer["n_points"]["noise"]["type-2"]   
+
         unique_labels = np.unique(labels)
         num_of_clusters = len(unique_labels)
-                  
+       
         fake_clusters = 0 #otlier = -1 and weak points that fail to merge (noise) = 0
         
-        i= 1
+        """"""
+        min_label = unique_labels.min()
+        i=  0
         for n in (unique_labels):
-            if n >= 0:
+
+            if n  >= 0: # num_of_clusters:
                 labels = np.where(labels==n, i, labels) 
-                i +=1
+                i +=1 
             else:
                 fake_clusters+=1
-               
+              
+
+    
         self.analyzer["n_clusters"]["detected"]=   num_of_clusters - fake_clusters  
         
         return  labels       
@@ -703,13 +784,13 @@ class DenMune():
     def plot_clusters(self, data_type=None, show_noise=False, show_plots=True):
         data2 = []
         colors2 = []
-
+     
         labels = self.preplot_Clusters(data_type=data_type)
-        
+
         if show_plots:
-          
-          palette = sns.color_palette( 'deep', np.unique(labels).max()+2) #deep, dark, bright, muted, pastel, colorblind
-          
+                  
+          palette = sns.color_palette( 'deep',  np.unique(labels).max()+2 ) #deep, dark, bright, muted, pastel, colorblind
+                  
           if self.prop_step:
               colors = [palette[x] if x > 0 else ( (0.0, 0.0, 0.0) if x == -1 else (0.0, 0.0, 0.0)) for x in labels]
               v = 0
@@ -731,7 +812,7 @@ class DenMune():
           if self.prop_step:
             if data_type == 'train':
               plt.scatter(data2[:self.train_sz].T[0], data2[:self.train_sz].T[1], c=colors2, **plot_kwds, marker='o')
-            elif data_type == 'train':
+            elif data_type == 'test':
               plt.scatter(data2[self.train_sz:].T[0], data2[self.train_sz:].T[1], c=colors2, **plot_kwds, marker='o') 
             elif data_type == 'augmented':
               plt.scatter(data2.T[0], data2.T[1], c=colors2, **plot_kwds, marker='o')   
@@ -742,7 +823,10 @@ class DenMune():
                 plt.scatter(self.data[self.train_sz:].T[0], self.data[self.train_sz:].T[1], c=colors, **plot_kwds, marker='o')
               elif data_type == 'augmented':
                 plt.scatter(self.data.T[0], self.data.T[1], c=colors, **plot_kwds, marker='o')
+              elif data_type == 'ground':
+                plt.scatter(self.data.T[0], self.data.T[1], c=colors, **plot_kwds, marker='o')  
         
+          self.colors = colors
           frame = plt.gca()
           frame.axes.get_xaxis().set_visible(False)
           frame.axes.get_yaxis().set_visible(False)
@@ -762,7 +846,7 @@ class DenMune():
         for i in range(data_len):
             for n in range(d):
                 mystr += str(round(dim_two[i][n],6))
-                if (n  < d-1):mystr += self.delimiter
+                if (n  < d-1):mystr += ','
                 if (n  == d-1): mystr += '\n'
 
         
@@ -805,5 +889,4 @@ class DenMune():
                         for z in subsubdic:
                             #print('4', 'key:', z , 'value:', subsubdic[z], 'parent:', v)
                             creat_TreefromDict(self, tree, subsubdic, z, parent=v)
-        tree.show()    
-        
\ No newline at end of file
+        tree.show()    
\ No newline at end of file
